/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * MoleculesCommandLineQuery.cpp
 *
 *  Created on: Oct 25, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Helpers/MemDebug.hpp"

#include <Descriptors/MoleculeDescriptor.hpp>
#include <Descriptors/MoleculeIdDescriptor.hpp>
#include "CommandLineUI/CommandLineDialog.hpp"

#include "Helpers/Log.hpp"
#include "Helpers/Verbose.hpp"
#include "CommandLineParser.hpp"
#include "World.hpp"

CommandLineDialog::MoleculesCommandLineQuery::MoleculesCommandLineQuery(string title, string _description) :
    Dialog::MoleculesQuery(title, _description)
{}

CommandLineDialog::MoleculesCommandLineQuery::~MoleculesCommandLineQuery() {}

bool CommandLineDialog::MoleculesCommandLineQuery::handle() {
  std::vector<int> IdxOfMol;
  if (CommandLineParser::getInstance().vm.count(getTitle())) {
    IdxOfMol = CommandLineParser::getInstance().vm[getTitle()].as< std::vector<int> >();
    for (std::vector<int>::iterator iter = IdxOfMol.begin(); iter != IdxOfMol.end(); ++iter) {
      temp = World::getInstance().getMolecule(MoleculeById(*iter));
      if (temp)
        tmp.push_back(temp);
    }
    return true;
  } else {
    DoeLog(1) && (eLog() << Verbose(1) << "CommandLineUI parsing error: Missing molecules for " << getTitle() << "." << endl);
    return false;
  }
}

