/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * BoxQtQuery.cpp
 *
 *  Created on: Oct 25, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <Qt/qboxlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qtablewidget.h>

#include "Helpers/MemDebug.hpp"

#include "UIElements/Qt4/QtDialog.hpp"

#include "LinearAlgebra/Matrix.hpp"
#include "World.hpp"

QtDialog::BoxQtQuery::BoxQtQuery(string _title,QBoxLayout *_parent,QtDialog *_dialog) :
    Dialog::BoxQuery(_title),
    parent(_parent)
{
  thisLayout = new QHBoxLayout();
  titleLabel = new QLabel(QString(getTitle().c_str()));

  // init input table
  inputTable = new QTableWidget(3,3, _dialog);
  QStringList CoordinateList;
  CoordinateList << "x" << "y" << "z";
  inputTable->setHorizontalHeaderLabels(CoordinateList);
  inputTable->setVerticalHeaderLabels(CoordinateList);

  pipe = new BoxQtQueryPipe(tmp,_dialog, inputTable);

  // fill the box with current matrix
  const Matrix &domain = World::getInstance().getDomain().getM();
  for (int i=0;i<3;i++)
    for (int j=0;j<3;j++) {
      QTableWidgetItem *newItem = new QTableWidgetItem(tr("%1").arg(domain.at(i,j)));
      inputTable->setItem(i,j,newItem);
      pipe->update(i,j);
    }

  parent->addLayout(thisLayout);
  thisLayout->addWidget(titleLabel);
  thisLayout->addWidget(inputTable);

  connect(inputTable,SIGNAL(cellChanged(int,int)),pipe,SLOT(update(int,int)));
}

QtDialog::BoxQtQuery::~BoxQtQuery()
{
  delete pipe;
}

bool QtDialog::BoxQtQuery::handle() {
  return true;
}


